@@ -54,6 +54,8 @@ class ChangesetEntry(object):
         else:
             s += ' from ' + self.old_name
         s += ')'
+        if isinstance(s, unicode):
+            s = s.encode('ascii', 'replace')
         return s
 
 
@@ -154,7 +156,7 @@ class Changeset(object):
         if self.REFILL_MESSAGE:
             self.log = refill(log)
         else:
-            self.log = log.strip()
+            self.log = log
 
     def addEntry(self, entry, revision, before=None):
         """
