@@ -89,7 +89,7 @@ class CvspsRepository(Repository):
             return
 
         makedirs(path)
-        cmd = self.command("-d", path, "init")
+        cmd = self.command("-f", "-d", path, "init")
         c = ExternalCommand(command=cmd)
         c.execute()
         if c.exit_status:
@@ -98,7 +98,7 @@ class CvspsRepository(Repository):
 
         if self.module:
             tempwc = mkdtemp('cvs', 'tailor')
-            cmd = self.command("-d", path, "import",
+            cmd = self.command("-f", "-d", path, "import",
                                "-m", "This directory will host the "
                                "upstream sources",
                                self.module, "tailor", "start")
@@ -248,7 +248,7 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
         if sincerev:
             sincerev = int(sincerev)
 
-        cmd = self.repository.command("--cvs-direct", "-u", "-b", branch,
+        cmd = self.repository.command("--norc", "--cvs-direct", "-u", "-b", branch,
                                       "--root", self.repository.repository,
                                       cvsps=True)
         cvsps = ExternalCommand(command=cmd)
@@ -259,13 +259,13 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
 
     def __maybeDeleteDirectory(self, entrydir, changeset):
         from os.path import join, exists
-        from os import listdir
+        from vcpx.repository.cvs import CvsEntries
 
         if not entrydir:
             return
 
         absentrydir = join(self.repository.basedir, entrydir)
-        if not exists(absentrydir) or listdir(absentrydir) == ['CVS']:
+        if not exists(absentrydir) or CvsEntries(absentrydir).isEmpty():
             # Oh, the directory is empty: if there are no other added entries
             # in the directory, insert a REMOVE event against it.
             for added in changeset.addedEntries():
@@ -280,7 +280,7 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
         from os import listdir
         from shutil import rmtree
         from time import sleep
-        from vcpx.repository.cvs import CvsEntries
+        from vcpx.repository.cvs import CvsEntries, compare_cvs_revs
 
         entries = CvsEntries(self.repository.basedir)
 
@@ -288,6 +288,9 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
         addeddirs = []
         deleteddirs = []
 
+        # Group entries at the same revision
+        reventries = {}
+
         for e in changeset.entries:
             if e.action_kind == e.UPDATED:
                 info = entries.getFileInfo(e.name)
@@ -325,53 +328,63 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
             # just a single "CVS" subdir, btw)
 
             if e.action_kind == e.DELETED and e.new_revision is None:
-                assert listdir(join(self.repository.basedir, e.name)) == ['CVS'], '%s should be empty' % e.name
+                assert CvsEntries(join(self.repository.basedir, e.name)).isEmpty(), '%s should be empty' % e.name
                 rmtree(join(self.repository.basedir, e.name))
             else:
-                cmd = self.repository.command("-d", "%(repository)s",
-                                              "-q", "update", "-d",
-                                              "-r", e.new_revision)
-                if self.repository.freeze_keywords:
-                    cmd.append('-kk')
-                cvsup = ExternalCommand(cwd=self.repository.basedir, command=cmd)
-                retry = 0
-                while True:
-                    cvsup.execute(e.name, repository=self.repository.repository)
-
-                    if cvsup.exit_status:
-                        retry += 1
-                        if retry>3:
-                            break
-                        delay = 2**retry
-                        self.log.warning("%s returned status %s, "
-                                         "retrying in %d seconds...",
-                                         str(cvsup), cvsup.exit_status, delay)
-                        sleep(retry)
-                    else:
-                        break
-
-                if cvsup.exit_status:
-                    raise ChangesetApplicationFailure(
-                        "%s returned status %s" % (str(cvsup),
-                                                   cvsup.exit_status))
-
-                self.log.debug("%s updated to %s", e.name, e.new_revision)
+                names = reventries.setdefault(e.new_revision, [])
+                names.append(e.name)
 
             if e.action_kind == e.DELETED:
                 entrydir = split(e.name)[0]
                 if self.__maybeDeleteDirectory(entrydir, changeset):
                     deleteddirs.append(entrydir)
 
+        revs = reventries.keys()
+        revs.sort(compare_cvs_revs)
+
+        cmd = self.repository.command("-f", "-d", "%(repository)s",
+                                      "-q", "update", "-d",
+                                      "-r", "%(revision)s")
+        if self.repository.freeze_keywords:
+            cmd.append('-kk')
+        cvsup = ExternalCommand(cwd=self.repository.basedir, command=cmd)
+        for rev in revs:
+            names = reventries[rev]
+            retry = 0
+            while True:
+                cvsup.execute(names, repository=self.repository.repository,
+                              revision=rev)
+                if cvsup.exit_status:
+                    retry += 1
+                    if retry>3:
+                        break
+                    delay = 2**retry
+                    self.log.warning("%s returned status %s, "
+                                     "retrying in %d seconds...",
+                                     str(cvsup), cvsup.exit_status, delay)
+                    sleep(retry)
+                else:
+                    break
+
+            if cvsup.exit_status:
+                raise ChangesetApplicationFailure(
+                    "%s returned status %s" % (str(cvsup),
+                                               cvsup.exit_status))
+
+            self.log.debug("%s updated to %s", ','.join(names), e.new_revision)
+
         # Fake up ADD and DEL events for the directories implicitly
         # added/removed, so that the replayer gets their name.
 
         for entry,path in addeddirs:
             entry = changeset.addEntry(path, None, before=entry)
             entry.action_kind = entry.ADDED
+            self.log.info("registering new %s directory", entry.name)
 
         for path in deleteddirs:
             deldir = changeset.addEntry(path, None)
             deldir.action_kind = deldir.DELETED
+            self.log.info("registering %s directory deletion", path)
 
     def _checkoutUpstreamRevision(self, revision):
         """
@@ -417,7 +430,7 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
             # CVS does not handle "checkout -d multi/level/subdir", so
             # split the basedir and use it's parentdir as cwd below.
             parentdir, subdir = split(self.repository.basedir)
-            cmd = self.repository.command("-q",
+            cmd = self.repository.command("-f", "-q",
                                           "-d", self.repository.repository,
                                           "checkout",
                                           "-d", subdir)
@@ -472,6 +485,13 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
             "Loop over changeset entries to determine if it's already applied."
 
             applied = False
+
+            # applied become True when an entry is DELETED *and* there is
+            # no metainfo for it: thus, a changeset that removes a few entries
+            # very late in history would be assumed as applied. Prevent that
+            # by checking for at least one explicit match on an existing entry.
+            onepositive = False
+
             for m in cs.entries:
                 info = entries.getFileInfo(m.name)
 
@@ -489,17 +509,23 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
                     # changeset is new.
                     if not applied:
                         break
+                    else:
+                        onepositive = True
                 elif m.action_kind == ChangesetEntry.DELETED:
                     applied = True
-            return applied
+            return applied and onepositive
+
+        # We cannot stop at the first not-applied cset, because it may
+        # old enough to trick already_applied(): an entry may have
+        # been moved in the meantime, and thus the getFileInfo()
+        # method would return None, for example... So we really have
+        # to loop over the whole queue.
 
         for cset in self.state_file:
             applied = already_applied(cset)
             found = found or applied
             if applied:
                 last = cset
-            else:
-                break
 
         if not found and initialcset:
             found = already_applied(initialcset)
@@ -597,7 +623,7 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
         if not self.repository.repository or exists(join(self.repository.basedir, 'CVS')):
             return
 
-        cmd = self.repository.command("-d", self.repository.repository, "co",
+        cmd = self.repository.command("-f", "-d", self.repository.repository, "co",
                                       "-d", self.repository.basedir)
         cvsco = ExternalCommand(command=cmd)
         cvsco.execute(self.repository.module)
@@ -635,7 +661,7 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
         Add some new filesystem objects.
         """
 
-        cmd = self.repository.command('-q', 'add', '-ko')
+        cmd = self.repository.command("-f", '-q', 'add', '-ko')
         ExternalCommand(cwd=self.repository.basedir, command=cmd).execute(names)
 
     def __forceTagOnEachEntry(self):
@@ -684,7 +710,8 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
                 remove(efn+'.tailor-old')
 
 
-    def _commit(self, date, author, patchname, changelog=None, entries=None):
+    def _commit(self, date, author, patchname, changelog=None, entries=None,
+                tags = [], isinitialcommit = False):
         """
         Commit the changeset.
         """
@@ -707,7 +734,7 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
         log.write(encode('\n'.join(logmessage)))
         log.close()
 
-        cmd = self.repository.command("-q", "ci", "-F", rontf.name)
+        cmd = self.repository.command("-f", "-q", "ci", "-F", rontf.name)
         if not entries:
             entries = ['.']
 
@@ -723,7 +750,7 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
         Remove some filesystem objects.
         """
 
-        cmd = self.repository.command("-q", "remove")
+        cmd = self.repository.command("-f", "-q", "remove")
         ExternalCommand(cwd=self.repository.basedir, command=cmd).execute(names)
 
     def _renamePathname(self, oldname, newname):
@@ -758,7 +785,7 @@ class CvspsWorkingDir(UpdatableSourceWorkingDir,
         if not iscvsalpha(tagname[0]):
             tagname = 'tag-' + tagname
 
-        cmd = self.repository.command("tag")
+        cmd = self.repository.command("-f", "tag")
         c = ExternalCommand(cwd=self.repository.basedir, command=cmd)
         c.execute(tagname)
         if c.exit_status:
