@@ -42,8 +42,9 @@ class DarcsChangeset(Changeset):
     def addEntry(self, entry, revision):
         """
         Fixup darcs idiosyncrasies:
-        
+
         - collapse "add A; rename A B" into "add B"
+        - annihilate "add A; remove A"
         - collapse "rename A B; remove B" into "remove A"
         """
 
@@ -55,7 +56,7 @@ class DarcsChangeset(Changeset):
         # Ok, before adding this entry, check it against already
         # known: if this is an add, and there's a rename (such as "add
         # A; rename A B; ") then...
-        
+
         if entry.action_kind == entry.ADDED:
             # ... we have to check existings, because of a bug in
             # darcs: `changes --xml` (as of 1.0.7) emits the changes
@@ -68,7 +69,7 @@ class DarcsChangeset(Changeset):
             # to an addition instead, and don't insert any other entry
 
             dirname = entry.name+'/' # darcs hopefully use forward slashes also under win
-            
+
             for i,e in enumerate(self.entries):
                 if e.action_kind == e.RENAMED and e.old_name == entry.name:
                     # Luckily enough (since removes are the first entries
@@ -88,18 +89,32 @@ class DarcsChangeset(Changeset):
                     self.entries.insert(i, entry)
                     return entry
 
-        # Likewise, if this is a deletion, and there is a rename of this
-        # entry (such as "rename A B; remove B") then ...
-        
+        # Likewise, if this is a deletion, and there is a rename of
+        # this entry (such as "rename A B; remove B") then turn the
+        # existing rename into a deletion instead.
+
+        # If instead the removed entry was added by the same patch,
+        # annihilate the two: a bug in darcs (possibly fixed in recent
+        # versions) created patches with ADD+EDIT+REMOVE of a single
+        # file (see tailor ticket #71, or darcs issue185). Too bad
+        # another bug (still present in 1.0.8) hides that and makes
+        # very hard (read: impossible) any workaround on the tailor
+        # side. Luckily I learnt another tiny bit of Haskell and
+        # proposed a fix for that: hopefully the patch will be
+        # accepted by darcs developers. In the meantime, I attached it
+        # to ticket #71: without that, tailor does not have enough
+        # information to do the right thing.
+
         elif entry.action_kind == entry.DELETED:
-            # turn the existing rename into a deletion instead
-            
             for i,e in enumerate(self.entries):
                 if e.action_kind == e.RENAMED and e.name == entry.name:
                     e.action_kind = e.DELETED
                     e.name = e.old_name
                     e.old_name = None
                     return e
+                elif e.action_kind == e.ADDED and e.name == entry.name:
+                    del self.entries[i]
+                    return None
 
         # Ok, it must be either an edit or a rename: the former goes
         # obviously to the end, and since the latter, as said, come
@@ -107,7 +122,7 @@ class DarcsChangeset(Changeset):
         self.entries.append(entry)
         return entry
 
-        
+
 def changesets_from_darcschanges(changes, unidiff=False, repodir=None,
                                  chunksize=2**15):
     """
@@ -122,7 +137,6 @@ def changesets_from_darcschanges(changes, unidiff=False, repodir=None,
     csets = changesets_from_darcschanges_unsafe(changes, unidiff,
                                                 repodir, chunksize)
     for cs in csets:
-        cs.tags = None
         yield cs
 
 def changesets_from_darcschanges_unsafe(changes, unidiff=False, repodir=None,
@@ -191,7 +205,7 @@ def changesets_from_darcschanges_unsafe(changes, unidiff=False, repodir=None,
                                       tags=self.current.get('tags',[]))
                 cset.darcs_hash = self.current['hash']
                 if self.darcsdiff:
-                    cset.unidiff = self.darcsdiff.execute(
+                    cset.unidiff = self.darcsdiff.execute(TZ='UTC',
                         stdout=PIPE, patchname=cset.revision)[0].read()
 
                 self.changesets.append(cset)
@@ -250,11 +264,6 @@ class DarcsSourceWorkingDir(UpdatableSourceWorkingDir):
         Do the actual work of fetching the upstream changeset.
         """
 
-        from datetime import datetime
-        from time import strptime
-        from sha import new
-        from vcpx.changes import Changeset
-
         cmd = self.repository.command("pull", "--dry-run")
         pull = ExternalCommand(cwd=self.repository.basedir, command=cmd)
         output = pull.execute(self.repository.repository,
@@ -265,6 +274,17 @@ class DarcsSourceWorkingDir(UpdatableSourceWorkingDir):
                 "%s returned status %d saying\n%s" %
                 (str(pull), pull.exit_status, output.read()))
 
+        return self._parseDarcsPull(output)
+
+
+    def _parseDarcsPull(self, output):
+        """Process 'darcs pull' output to Changesets.
+        """
+        from datetime import datetime
+        from time import strptime
+        from sha import new
+        from vcpx.changes import Changeset
+
         l = output.readline()
         while l and not (l.startswith('Would pull the following changes:') or
                          l == 'No remote changes to pull in!\n'):
@@ -279,21 +299,32 @@ class DarcsSourceWorkingDir(UpdatableSourceWorkingDir):
                 # Assume it's a line like
                 #    Sun Jan  2 00:24:04 UTC 2005  lele@nautilus.homeip.net
                 # we used to split on the double space before the email,
-                # but in this case this is wrong. Waiting for xml output,
-                # is it really sane asserting date's length to 28 chars?
-                date = l[:28]
-                author = l[30:-1]
+                # but in this case this is wrong. Then we assumed the date
+                # part to be exactly 28 chars long, but what about timezone
+                # names like 'CEST'? Waiting for xml output...
+                # We still assume there are *two* spaces before the email.
+                # The alternative is using some sort of a regex: Aaron Kaplan
+                # kindly suggested his own perl snippet
+                #    /^(... ... .\d .\d:\d\d:\d\d ...?. \d\d\d\d)  (.*)/ || die;
+                #    my ($date, $author) = ($1, $2);
+                # but that assumes the two spaces as separator, so I find the
+                # following solution easier and by any chance faster too.                
+                pieces = l.rstrip().split('  ')
+                assert len(pieces)>1, "Cannot parse %r as a patch timestamp" % l
+                author = pieces.pop()
+                date = ' '.join(pieces)
                 y,m,d,hh,mm,ss,d1,d2,d3 = strptime(date, "%a %b %d %H:%M:%S %Z %Y")
                 date = datetime(y,m,d,hh,mm,ss,0,UTC)
-                l = output.readline()
+                l = output.readline().rstrip()
                 assert (l.startswith('  * ') or
                         l.startswith('  UNDO:') or
-                        l.startswith('  tagged'))
+                        l.startswith('  tagged')), \
+                        "Got %r but expected the start of the log" % l
 
                 if l.startswith('  *'):
-                    name = l[4:-1]
+                    name = l[4:]
                 else:
-                    name = l[2:-1]
+                    name = l[2:]
 
                 changelog = []
                 l = output.readline()
@@ -308,6 +339,15 @@ class DarcsSourceWorkingDir(UpdatableSourceWorkingDir):
                     inverted = 't'
                 else:
                     inverted = 'f'
+
+                if name.startswith('tagged '):
+                    name = name[7:]
+                    if cset.tags is None:
+                        cset.tags = [name]
+                    else:
+                        cset.tags.append(name)
+                    name = "TAG " + name
+
                 phash = new()
                 phash.update(name)
                 phash.update(author)
@@ -318,11 +358,8 @@ class DarcsSourceWorkingDir(UpdatableSourceWorkingDir):
                                                    new(author).hexdigest()[:5],
                                                    phash.hexdigest())
 
-                if name.startswith('tagged'):
-                    self.log.warning("Skipping tag %s because I don't "
-                                     "propagate tags from darcs.", name)
-                else:
-                    yield cset
+
+                yield cset
 
                 while not l.strip():
                     l = output.readline()
@@ -371,10 +408,10 @@ class DarcsSourceWorkingDir(UpdatableSourceWorkingDir):
         line = output.readline()
         while line:
             if line.startswith('We have conflicts in the following files:'):
-                files = output.readline()[:-1].split('./')[1:]
+                files = output.readline()[:-1].split(' ')
                 self.log.warning("Conflict after 'darcs pull': %s",
                                  ' '.join(files))
-                conflicts.extend(['./' + f for f in files])
+                conflicts.extend(files)
             line = output.readline()
 
         cmd = self.repository.command("changes", selector, revtag,
