@@ -30,6 +30,12 @@ function get_passphrase(KEYPAIR_ID)
 end
 """
 
+class Unknown(object):
+    '''
+    Assign this class to variables whose value is unknown.
+    Similar to the "None" object.
+    '''
+
 class MonotoneRepository(Repository):
     METADIR = '_MTN'
 
@@ -41,12 +47,70 @@ class MonotoneRepository(Repository):
                      cget(self.name, '%s-keyid' % self.which)
         self.passphrase = cget(self.name, 'passphrase') or \
                           cget(self.name, '%s-passphrase' % self.which)
-        self.keyfile = cget(self.name, 'keyfile') or \
-                       cget(self.name, '%s-keyfile' % self.which)
         self.keygenid = cget(self.name, 'keygenid') or \
                         cget(self.name, '%s-keygenid' % self.which)
         self.custom_lua = cget(self.name, 'custom_lua') or \
                           cget(self.name, '%s-custom_lua' % self.which)
+        self._supportsLogRevision = Unknown
+        self._supportsLogNoGraph = Unknown
+
+    def supportsLogRevision(self, working_dir, revision):
+        """
+        In Monotone 0.32, ``mtn log --revision`` was changed to ``mtn
+        log --from``.  This function returns True if ``--revision``
+        is supported or False otherwise.
+
+        FIXME: To call this method, you have to actually pass working_dir
+        and a revision to it.  Ideally, this is unnecessary and it is
+        really just a hack.  Perhaps a better way to do this is to have a
+        ``log`` method on the MonotoneRepository class.
+        """
+
+        # Lazily (upon first request) attempt to determine the value
+        # of this variable:
+        if self._supportsLogRevision is Unknown:
+            cmd = self.command("log",
+                               "--db", self.repository,
+                               "--last", "1",
+                               "--revision", revision)
+            mtl = ExternalCommand(cwd=working_dir, command=cmd)
+            outstr = mtl.execute(stdout=PIPE, stderr=PIPE)
+            if mtl.exit_status:
+                self._supportsLogRevision = False
+            else:
+                self._supportsLogRevision = True
+
+        return self._supportsLogRevision
+
+    def supportsLogNoGraph(self, working_dir, revision):
+        """
+        In Monotone 0.33, the log message format was changed and the
+        ``mtn log --no-graph`` option was added to revert to the prior log
+        format.  Since this new format is intended for pretty-printing,
+        it is not very helpful when screen-scraping.
+
+        FIXME: (see the FIXME in supportsLogRevision)
+        """
+
+        # Lazily (upon first request) attempt to determine the value
+        # of this variable:
+        if self._supportsLogNoGraph is Unknown:
+            if not self.supportsLogRevision(working_dir, revision):
+                cmd = self.command("log",
+                                   "--db", self.repository,
+                                   "--last", "1",
+                                   "--from", revision,
+                                   "--no-graph") # <- we're testing for this!
+                mtl = ExternalCommand(cwd=working_dir, command=cmd)
+                outstr = mtl.execute(stdout=PIPE, stderr=PIPE)
+                if mtl.exit_status:
+                    self._supportsLogNoGraph = False
+                else:
+                    self._supportsLogNoGraph = True
+            else:
+                self._supportsLogNoGraph = False
+
+        return self._supportsLogNoGraph
 
     def create(self):
         """
@@ -58,7 +122,7 @@ class MonotoneRepository(Repository):
 
         cmd = self.command("db", "init", "--db", self.repository)
         init = ExternalCommand(command=cmd)
-        init.execute()
+        init.execute(stdout=PIPE, stderr=PIPE)
 
         if init.exit_status:
             raise TargetInitializationFailure("Was not able to initialize "
@@ -69,33 +133,26 @@ class MonotoneRepository(Repository):
             self.log.info("Using key %s for commits" % (self.keyid,))
         else:
             # keystore key id unspecified, look at other options
-            if self.keyfile:
-                # a key file is available, read into the database
-                keyfile = file(self.keyfile)
-                cmd = self.repository.command("read", "--db",
-                                              self.repository)
-                regkey = ExternalCommand(command=cmd)
-                regkey.execute(input=keyfile)
-            elif self.keygenid:
+            if self.keygenid:
                 # requested a new key
-                cmd = self.repository.command("genkey", "--db",
-                                              self.repository)
+                cmd = self.command("genkey", "--db", self.repository)
                 regkey = ExternalCommand(command=cmd)
                 if self.passphrase:
-                    passp="%s\n%s\n" % (self.passphrase,
-                                        self.passphrase)
-                regkey.execute(self.keygenid, input=passp)
+                    passp = "%s\n%s\n" % (self.passphrase, self.passphrase)
+                else:
+                    passp = None
+                regkey.execute(self.keygenid, input=passp, stdout=PIPE, stderr=PIPE)
+                if regkey.exit_status:
+                    raise TargetInitializationFailure("Was not able to setup "
+                                                  "the monotone initial key at %r" %
+                                                  self.repository)
             else:
                 raise TargetInitializationFailure("Can't setup the monotone "
                                                   "repository %r. "
-                                                  "A keyid, keyfile or keygenid "
+                                                  "A keyid or keygenid "
                                                   "must be provided." %
                                                   self.repository)
 
-            if regkey.exit_status:
-                raise TargetInitializationFailure("Was not able to setup "
-                                                  "the monotone initial key at %r" %
-                                                  self.repository)
 
 
 class ExternalCommandChain:
@@ -125,6 +182,7 @@ class ExternalCommandChain:
             outstr = out.getvalue()
         return out, err
 
+
 class MonotoneChangeset(Changeset):
     """
     Monotone changesets differ from standard Changeset because:
@@ -169,6 +227,7 @@ class MonotoneChangeset(Changeset):
         self.real_ancestors = real_ancestors
         self.branches = branches
 
+
 class MonotoneLogParser:
     """
     Obtain and parse a *single* "mtn log" output, reconstructing
@@ -213,10 +272,30 @@ class MonotoneLogParser:
         self.changelog=""
         self.branches=[]
 
-        cmd = self.repository.command("log", "--db", self.repository.repository,
-                                      "--last", "1", "--revision", revision)
+        cmd = None
+
+        logRevision = self.repository.supportsLogRevision(self.working_dir,
+                                                          revision)
+        logNoGraph = self.repository.supportsLogNoGraph(self.working_dir,
+                                                        revision)
+        if logRevision: # <= Montone-0.31
+            cmd = self.repository.command("log",
+                                          "--db", self.repository.repository,
+                                          "--last", "1",
+                                          "--revision", revision)
+        elif not logRevision and not logNoGraph: # Monotone-0.32
+            cmd = self.repository.command("log",
+                                          "--db", self.repository.repository,
+                                          "--last", "1",
+                                          "--from", revision)
+        elif not logRevision and logNoGraph: # Monotone-0.33
+            cmd = self.repository.command("log",
+                                          "--db", self.repository.repository,
+                                          "--last", "1",
+                                          "--from", revision,
+                                          "--no-graph")
         mtl = ExternalCommand(cwd=self.working_dir, command=cmd)
-        outstr = mtl.execute(stdout=PIPE)
+        outstr = mtl.execute(stdout=PIPE, stderr=PIPE)
         if mtl.exit_status:
             raise GetUpstreamChangesetsFailure("mtn log returned status %d" % mtl.exit_status)
 
@@ -229,7 +308,8 @@ class MonotoneLogParser:
             pr = self.PrefixRemover(curline)
             if pr("Revision:"):
                 if pr.value != revision:
-                    raise GetUpstreamChangesetsFailure("Revision doesn't match. Expected %s, found %s" % revision, pr.value)
+                    raise GetUpstreamChangesetsFailure(
+                        "Revision doesn't match. Expected %s, found %s" % (revision, pr.value))
                 state = self.SINGLE
             elif pr("Ancestor:"):
                 if pr.value:
@@ -300,6 +380,7 @@ class MonotoneLogParser:
 
         return chset
 
+
 class MonotoneDiffParser:
     """
     This class obtains a diff beetween two arbitrary revisions, parsing
@@ -408,7 +489,7 @@ class MonotoneDiffParser:
                                       "--revision", chset.revision)
 
         mtl = ExternalCommand(cwd=self.working_dir, command=cmd)
-        outstr = mtl.execute(stdout=PIPE)
+        outstr = mtl.execute(stdout=PIPE, stderr=PIPE)
         if mtl.exit_status:
             raise GetUpstreamChangesetsFailure(
                 "mtn diff returned status %d" % mtl.exit_status)
@@ -429,9 +510,12 @@ class MonotoneDiffParser:
                     break
                 else:
                     in_item = False
-                    # now, next token should be a string or an hash
+                    # now, next token should be a string or an hash,
+                    # or the two tokens are "no changes"
                     fname = tkiter.next()
-                    if fname[0] != '"' and fname[0] != '[':
+                    if token == "no" and fname == "changes":
+                        break
+                    elif fname[0] != '"' and fname[0] != '[':
                         raise GetUpstreamChangesetsFailure(
                             "Unexpected token sequence: '%s' "
                             "followed by '%s'" %(token, fname))
@@ -476,7 +560,6 @@ class MonotoneDiffParser:
                 raise GetUpstreamChangesetsFailure("Unexpected end of 'diff' parsing changeset info")
 
 
-
 class MonotoneRevToCset:
     """
     This class is used to create changesets from revision ids.
@@ -586,7 +669,7 @@ class MonotoneWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingD
             cmd = self.repository.command("automate","heads",
                                           "--db", dbrepo, module)
             mtl = ExternalCommand(cwd=working_dir, command=cmd)
-            outstr = mtl.execute(stdout=PIPE)
+            outstr = mtl.execute(stdout=PIPE, stderr=PIPE)
             if mtl.exit_status:
                 raise InvocationError("The branch '%s' is empty" % module)
 
@@ -666,7 +749,7 @@ class MonotoneWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingD
     def _applyChangeset(self, changeset):
         cmd = self.repository.command("update", "--revision", changeset.revision)
         mtl = ExternalCommand(cwd=self.repository.basedir, command=cmd)
-        mtl.execute()
+        mtl.execute(stdout=PIPE, stderr=PIPE)
         if mtl.exit_status:
             raise ChangesetApplicationFailure("'mtn update' returned "
                                               "status %s" % mtl.exit_status)
@@ -694,7 +777,7 @@ class MonotoneWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingD
                                           "--branch", self.repository.module,
                                           self.repository.basedir)
             mtl = ExternalCommand(cwd=self.repository.rootdir, command=cmd)
-            mtl.execute()
+            mtl.execute(stdout=PIPE, stderr=PIPE)
             if mtl.exit_status:
                 raise TargetInitializationFailure(
                     "'mtn co' returned status %s" % mtl.exit_status)
@@ -702,7 +785,6 @@ class MonotoneWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingD
             self.log.debug("%r already exists, assuming it's a monotone "
                            "working dir already populated", self.repository.basedir)
 
-
         # Ok, now the workdir contains the checked out revision. We
         # need to return a changeset describing it.  Since this is the
         # first revision checked out, we don't have a (linearized)
@@ -721,37 +803,42 @@ class MonotoneWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingD
 
     def _addPathnames(self, names):
         """
-        Add some new filesystem objects, skipping directories (directory
-        addition is implicit in monotone)
+        Add some new filesystem objects, skipping directories.
+        In monotone *explicit* directory addition is always recursive,
+        so adding a directory here might interfere with renames.
+        Adding files without directories doesn't cause problems,
+        because adding a file implicitly adds the parent directory
+        (non-recursively).
         """
         fnames=[]
         for fn in names:
             if isdir(join(self.repository.basedir, fn)):
-                self.log.debug("ignoring addition of directory %r", fn)
+                self.log.debug("ignoring addition of directory %r "
+                               "(dirs are implicitly added by files)", fn)
             else:
                 fnames.append(fn)
         if len(fnames):
             # ok, we still have something to add
-            cmd = self.repository.command("add")
+            cmd = self.repository.command("add", "--")
             add = ExternalCommand(cwd=self.repository.basedir, command=cmd)
-            add.execute(fnames)
+            add.execute(fnames, stdout=PIPE, stderr=PIPE)
             if add.exit_status:
                 raise ChangesetApplicationFailure("%s returned status %s" %
                                                     (str(add),add.exit_status))
 
-
     def _addSubtree(self, subdir):
         """
-        Add a whole subtree
+        Add a whole subtree (recursively)
         """
-        cmd = self.repository.command("add")
+        cmd = self.repository.command("add", "--recursive", "--")
         add = ExternalCommand(cwd=self.repository.basedir, command=cmd)
-        add.execute(subdir)
+        add.execute(subdir, stdout=PIPE, stderr=PIPE)
         if add.exit_status:
             raise ChangesetApplicationFailure("%s returned status %s" %
                                               (str(add),add.exit_status))
 
-    def _commit(self, date, author, patchname, changelog=None, entries=None):
+    def _commit(self, date, author, patchname, changelog=None, entries=None,
+                tags = [], isinitialcommit = False):
         """
         Commit the changeset.
         """
@@ -798,9 +885,9 @@ class MonotoneWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingD
         Remove some filesystem object.
         """
 
-        cmd = self.repository.command("drop")
+        cmd = self.repository.command("drop", "--recursive", "--")
         drop = ExternalCommand(cwd=self.repository.basedir, command=cmd)
-        dum, error = drop.execute(names, stderr=PIPE)
+        dum, error = drop.execute(names, stdout=PIPE, stderr=PIPE)
         if drop.exit_status:
             errtext = error.read()
             self.log.error("Monotone drop said: %s", errtext)
@@ -812,26 +899,12 @@ class MonotoneWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingD
         """
         Rename a filesystem object.
         """
-        # this function is called *after* the file/dir has changed name,
-        # and monotone doesn't like it.
-        # we put names back to make it happy ...
-#        if access(join(self.repository.basedir, newname), F_OK):
-#            if access(join(self.repository.basedir, oldname), F_OK):
-#                raise ChangesetApplicationFailure("Can't rename %s to %s. "
-#                                                  "Both names already exist" %
-#                                                  (oldname, newname))
-#            renames(join(self.repository.basedir, newname), join(self.repository.basedir, oldname))
-#            self.log.debug('Renamed "%s" back to "%s"', newname, oldname)
-
-        cmd = self.repository.command("rename")
+        cmd = self.repository.command("rename", "--")
         rename = ExternalCommand(cwd=self.repository.basedir, command=cmd)
-        rename.execute(oldname, newname)
-
-        # redo the rename ...
-#        renames(join(self.repository.basedir, oldname), join(self.repository.basedir, newname))
-#        if rename.exit_status:
-#            raise ChangesetApplicationFailure("%s returned status %s" %
-#                                              (str(rename),rename.exit_status))
+        rename.execute(oldname, newname, stdout=PIPE, stderr=PIPE)
+        if rename.exit_status:
+            raise ChangesetApplicationFailure(
+                     "%s returned status %s" % (str(rename),rename.exit_status))
 
     def _prepareTargetRepository(self):
         """
@@ -861,11 +934,13 @@ class MonotoneWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingD
                                       "--db", self.repository.repository,
                                       "--branch", self.repository.module)
 
+        if self.repository.keygenid:
+           self.repository.keyid = self.repository.keygenid
         if self.repository.keyid:
             cmd.extend( ("--key", self.repository.keyid) )
 
         setup = ExternalCommand(command=cmd)
-        setup.execute(self.repository.basedir)
+        setup.execute(self.repository.basedir, stdout=PIPE, stderr=PIPE)
 
         if self.repository.passphrase or self.repository.custom_lua:
             monotonerc = open(join(self.repository.basedir, '_MTN', 'monotonerc'), 'w')
@@ -894,7 +969,7 @@ class MonotoneWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingD
             ignored.append('^%s$' % escape(sfrelname + '.journal'))
 
         if len(ignored) > 0:
-            mt_ignored = open(join(self.repository.basedir, '.mtn-ignore'), 'aU')
+            mt_ignored = open(join(self.repository.basedir, '.mtn-ignore'), 'a')
             mt_ignored.write('\n'.join(ignored))
             mt_ignored.close()
 
