@@ -33,6 +33,7 @@ class SvnRepository(Repository):
         self.EXECUTABLE = cget(self.name, 'svn-command', 'svn')
         self.__svnadmin = cget(self.name, 'svnadmin-command', 'svnadmin')
         self.use_propset = cget(self.name, 'use-propset', False)
+        self.propset_date = cget(self.name, 'propset-date', True)
         self.filter_badchars = cget(self.name, 'filter-badchars', False)
         self.use_limit = cget(self.name, 'use-limit', True)
         self.trust_root = cget(self.name, 'trust-root', False)
@@ -70,10 +71,10 @@ class SvnRepository(Repository):
 
     def create(self):
         """
-        Create a local SVN repository, if it does not exist.
+        Create a local SVN repository, if it does not exist, and configure it.
         """
 
-        from os.path import join
+        from os.path import join, exists
         from sys import platform
 
         # Verify the existence of repository by listing its root
@@ -81,49 +82,52 @@ class SvnRepository(Repository):
         svnls = ExternalCommand(command=cmd)
         svnls.execute(self.repository)
 
-        # If it a valid repository, there's nothing else to do
-        if not svnls.exit_status:
-            return
+        # Create it if it isn't a valid repository
+        if svnls.exit_status:
+            if not self.repository.startswith('file:///'):
+                raise TargetInitializationFailure("%r does not exist and "
+                                                  "cannot be created since "
+                                                  "it's not a local (file:///) "
+                                                  "repository" %
+                                                  self.repository)
 
-        if not self.repository.startswith('file:///'):
-            raise TargetInitializationFailure("%r does not exist and "
-                                              "cannot be created since "
-                                              "it's not a local (file:///) "
-                                              "repository" %
-                                              self.repository)
+            repodir = self.repository[7:]
+            cmd = self.command("create", "--fs-type", "fsfs", svnadmin=True)
+            svnadmin = ExternalCommand(command=cmd)
+            svnadmin.execute(repodir)
 
-        repodir = self.repository[7:]
-        cmd = self.command("create", "--fs-type", "fsfs", svnadmin=True)
-        svnadmin = ExternalCommand(command=cmd)
-        svnadmin.execute(repodir)
-
-        if svnadmin.exit_status:
-            raise TargetInitializationFailure("Was not able to create a 'fsfs' "
-                                              "svn repository at %r" %
-                                              self.repository)
+            if svnadmin.exit_status:
+                raise TargetInitializationFailure("Was not able to create a 'fsfs' "
+                                                  "svn repository at %r" %
+                                                  self.repository)
         if self.use_propset:
             hookname = join(repodir, 'hooks', 'pre-revprop-change')
             if platform == 'win32':
                 hookname += '.bat'
-            prehook = open(hookname, 'wU')
-            if platform <> 'win32':
-                prehook.write('#!/bin/sh\n')
-            prehook.write('exit 0\n')
-            prehook.close()
-            if platform <> 'win32':
-                from os import chmod
-                chmod(hookname, 0755)
+            if not exists(hookname):
+                prehook = open(hookname, 'w')
+                if platform <> 'win32':
+                    prehook.write('#!/bin/sh\n')
+                prehook.write('exit 0\n')
+                prehook.close()
+                if platform <> 'win32':
+                    from os import chmod
+                    chmod(hookname, 0755)
 
         if self.module and self.module <> '/':
-            cmd = self.command("mkdir", "-m",
-                               "This directory will host the upstream sources")
-            svnmkdir = ExternalCommand(command=cmd)
-            svnmkdir.execute(self.repository + self.module)
-            if svnmkdir.exit_status:
-                raise TargetInitializationFailure("Was not able to create the "
-                                                  "module %r, maybe more than "
-                                                  "one level directory?" %
-                                                  self.module)
+            cmd = self.command("ls")
+            svnls = ExternalCommand(command=cmd)
+            svnls.execute(self.repository + self.module)
+            if svnls.exit_status:
+                cmd = self.command("mkdir", "-m",
+                                   "This directory will host the upstream sources")
+                svnmkdir = ExternalCommand(command=cmd)
+                svnmkdir.execute(self.repository + self.module)
+                if svnmkdir.exit_status:
+                    raise TargetInitializationFailure("Was not able to create the "
+                                                      "module %r, maybe more than "
+                                                      "one level directory?" %
+                                                      self.module)
 
 def changesets_from_svnlog(log, repository, chunksize=2**15):
     from xml.sax import make_parser
@@ -139,12 +143,11 @@ def changesets_from_svnlog(log, repository, chunksize=2**15):
         #   "/CMFPlone/branches/Plone-2_0-branch/tests/PloneTestCase.py"
         # that is to say "tests/PloneTestCase.py"
 
+        if not module.endswith('/'):
+            module = module + '/'
         if path.startswith(module):
             relative = path[len(module):]
-            if relative.startswith('/'):
-                return relative[1:]
-            else:
-                return relative
+            return relative
 
         # The path is outside our tracked tree...
         repository.log.warning('Ignoring %r since it is not under %r',
@@ -473,8 +476,9 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
             cmd.extend(["--revision", revision])
             svnco = ExternalCommand(command=cmd)
 
-            out, err = svnco.execute("%s%s" % (self.repository.repository,
-                                               self.repository.module),
+            out, err = svnco.execute("%s%s@%s" % (self.repository.repository,
+                                                  self.repository.module,
+                                                  revision),
                                      self.repository.basedir, stdout=PIPE, stderr=PIPE)
             if svnco.exit_status:
                 raise TargetInitializationFailure(
@@ -516,7 +520,8 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
                                       "--non-recursive")
         ExternalCommand(cwd=self.repository.basedir, command=cmd).execute(names)
 
-    def _commit(self, date, author, patchname, changelog=None, entries=None):
+    def _commit(self, date, author, patchname, changelog=None, entries=None,
+                tags = [], isinitialcommit = False):
         """
         Commit the changeset.
         """
@@ -538,6 +543,9 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
             logmessage.append('')
             logmessage.append('Original author: %s' % encode(author))
             logmessage.append('Date: %s' % date)
+        elif not self.repository.propset_date:
+            logmessage.append('')
+            logmessage.append('Date: %s' % date)
 
         rontf = ReopenableNamedTemporaryFile('svn', 'tailor')
         log = open(rontf.name, "w")
@@ -578,11 +586,11 @@ class SvnWorkingDir(UpdatableSourceWorkingDir, SynchronizableTargetWorkingDir):
             cmd = self.repository.command("propset", "%(propname)s",
                                           "--quiet", "--revprop",
                                           "--revision", revision)
-            propset = ExternalCommand(cwd=self.repository.basedir, command=cmd)
-
-            date = date.astimezone(UTC).replace(microsecond = 0, tzinfo=None)
-            propset.execute(date.isoformat()+".000000Z", propname='svn:date')
-            propset.execute(encode(author), propname='svn:author')
+            pset = ExternalCommand(cwd=self.repository.basedir, command=cmd)
+            if self.repository.propset_date:
+                date = date.astimezone(UTC).replace(microsecond=0, tzinfo=None)
+                pset.execute(date.isoformat()+".000000Z", propname='svn:date')
+            pset.execute(encode(author), propname='svn:author')
 
         cmd = self.repository.command("update", "--quiet")
         if self.repository.ignore_externals:
