@@ -11,8 +11,9 @@ Implement the frontend functionalities.
 
 __docformat__ = 'reStructuredText'
 
-__version__ = '0.9.26'
+__version__ = '0.9.28'
 
+from logging import getLogger
 from optparse import OptionParser, OptionGroup, Option
 from vcpx import TailorException
 from vcpx.config import Config, ConfigurationError
@@ -134,7 +135,7 @@ class Tailorizer(Project):
 
         pname_format = pconfig('patch-name-format', raw=True)
         if pname_format is not None:
-            SynchronizableTargetWorkingDir.PATCH_NAME_FORMAT = pname_format
+            SynchronizableTargetWorkingDir.PATCH_NAME_FORMAT = pname_format.strip() or '%(revision)s'
         SynchronizableTargetWorkingDir.REMOVE_FIRST_LOG_LINE = pconfig('remove-first-log-line')
         Changeset.REFILL_MESSAGE = pconfig('refill-changelogs')
 
@@ -178,7 +179,7 @@ GENERAL_OPTIONS = [
                 action="store_true", default=False,
                 help="Be verbose, echoing the changelog of each applied "
                      "changeset to stdout."),
-    RecogOption("--configfile", metavar="CONFNAME",
+    RecogOption("-c", "--configfile", metavar="CONFNAME",
                 help="Centralized storage of projects info.  With this "
                      "option and no other arguments tailor will update "
                      "every project found in the config file."),
@@ -261,7 +262,7 @@ BOOTSTRAP_OPTIONS = [
 ]
 
 VC_SPECIFIC_OPTIONS = [
-    RecogOption("--use-svn-propset", action="store_true", default=False,
+    RecogOption("--use-propset", action="store_true", default=False,
                 dest="use_propset",
                 help="Use 'svn propset' to set the real date and author of "
                      "each commit, instead of appending these information to "
@@ -360,7 +361,12 @@ def main():
         config.set('project', 'start-revision', options.start_revision)
 
         config.add_section(source)
-        config.set(source, 'repository', options.source_repository)
+        if options.source_repository:
+            config.set(source, 'repository', options.source_repository)
+        else:
+            logger = getLogger('tailor')
+            logger.warning("By any chance you forgot either the --source-repository or the --configfile option...")
+
         if options.source_module:
             config.set(source, 'module', options.source_module)
 
