@@ -123,7 +123,7 @@ class SynchronizableTargetWorkingDir(WorkingDir):
         previous = signal(SIGINT, SIG_IGN)
         try:
             self._commit(changeset.date, changeset.author, patchname, log,
-                         entries)
+                         entries, tags = changeset.tags)
             if changeset.tags:
                 for tag in changeset.tags:
                     self._tag(tag)
@@ -387,7 +387,8 @@ class SynchronizableTargetWorkingDir(WorkingDir):
                 self._addPathnames([join(dir, df)[len(self.repository.basedir)+1:]
                                     for df in subdirs + files])
 
-    def _commit(self, date, author, patchname, changelog=None, entries=None):
+    def _commit(self, date, author, patchname, changelog=None, entries=None,
+                tags = [], isinitialcommit = False):
         """
         Commit the changeset.
         """
@@ -544,7 +545,8 @@ class SynchronizableTargetWorkingDir(WorkingDir):
             author = "%s@%s" % (AUTHOR, HOST)
             patchname = BOOTSTRAP_PATCHNAME
             log = BOOTSTRAP_CHANGELOG % locals()
-        self._commit(changeset.date, author, patchname, log)
+        self._commit(changeset.date, author, patchname, log,
+                     isinitialcommit = True)
 
         if changeset.tags:
             for tag in changeset.tags:
