@@ -8,7 +8,7 @@ projects = project2
 [project1]
 root-directory = /tmp/tailor-tests
 source = svn:project1repo
-target = darcs:project1repo
+target = darcs:
 refill-changelogs = Yes
 state-file = project1.state
 before-commit = (maybe_skip, refill, p1_remap_authors)
@@ -19,7 +19,7 @@ repository = svn://some.server/svn
 module = project1
 use-propset = Yes
 
-[darcs:project1repo]
+[darcs:project1]
 repository = ~/darcs/project1
 
 [monotone:project1repo]
@@ -28,7 +28,7 @@ passphrase = simba
 
 [project2]
 root-directory = /tmp/tailor-tests
-source = darcs:project1repo
+source = darcs:project1
 target = svn:project2repo
 refill-changelogs = Yes
 state-file = project2.state
@@ -40,16 +40,15 @@ before-commit = refill
 root-directory = /tmp/tailor-tests
 source = svn:project3repo
 target = darcs:project3repo
+state-file = hidden
 
 [svn:project3repo]
 repository = svn://sample.org/svn
 module = /trunk
 subdir = plain
 
-[darcs:project3repo]
-subdir = .
-
 [project4]
+state-file = ~/tailorize/project4.state
 source = svn:project3repo
 target = darcs:project4repo
 
@@ -68,6 +67,18 @@ repository = /tmp/tailor-tests
 repository = /tmp/tailor-tests
 tag-entries = False
 
+[project6]
+root-directory = ~/tailorizedp
+state-file = hidden
+source = darcs:
+target = hg:
+
+[hg:project6]
+subdir = project6
+
+[darcs:project6]
+repository = ~/WiP/tailor
+
 '''
 
 def maybe_skip(context, changeset):
