@@ -13,7 +13,7 @@ target-module = None
 source-repository = %(tailor_repo)s
 encoding = None
 target-repository = None
-use-svn-propset = False
+use-propset = False
 source-module = None
 update = True
 subdir = test
@@ -289,6 +289,21 @@ repository = %(testdir)s/cvsdirtest.cvsrepo
 module = test
 repository = file://%(testdir)s/cvsdirtest.svnrepo
 
+[svnresurdirtest]
+target = svn:cvsresurdirtest
+start-revision = INITIAL
+root-directory = %(testdir)s/cvsresurdirtest/cvs2svn
+source = cvs:cvsresurdirtest
+subdir = test-work
+
+[cvs:cvsresurdirtest]
+module = test
+repository = %(testdir)s/cvsresurdirtest.cvsrepo
+
+[svn:cvsresurdirtest]
+module = test
+repository = file://%(testdir)s/cvsresurdirtest.svnrepo
+
 '''
 
 def remap_authors(context, changeset):
@@ -314,6 +329,7 @@ class OperationalTest(TestCase):
         from os.path import exists, split, join
         from atexit import register
         from shutil import rmtree
+        from vcpx.tests import DEBUG
 
         tailor_repo = getcwd()
         while tailor_repo != '/' and not exists(join(tailor_repo, '_darcs')):
@@ -321,7 +337,9 @@ class OperationalTest(TestCase):
         assert exists(join(tailor_repo, '_darcs')), "Tailor Darcs repository not found!"
         self.tailor_repo = tailor_repo
         self.config = Config(StringIO(__doc__), {'tailor_repo': tailor_repo,
-                                                 'testdir': self.TESTDIR})
+                                                 'testdir': self.TESTDIR,
+                                                 'verbose': DEBUG})
+
         if not exists(self.TESTDIR):
             mkdir(self.TESTDIR)
             register(rmtree, self.TESTDIR)
@@ -329,6 +347,8 @@ class OperationalTest(TestCase):
     def diffWhenPossible(self, tailorizer):
         "Diff the resulting sides"
 
+        from vcpx.tests import DEBUG
+
         dwd = tailorizer.workingDir()
         if not dwd.shared_basedirs:
             cmd = ["diff", "-r", "-u"]
@@ -336,7 +356,7 @@ class OperationalTest(TestCase):
                 cmd.extend(["-x", tailorizer.source.METADIR])
             if tailorizer.target.METADIR:
                 cmd.extend(["-x", tailorizer.target.METADIR])
-            d = ExternalCommand(command=cmd, nolog=True)
+            d = ExternalCommand(command=cmd, nolog=not DEBUG)
             out = d.execute(dwd.source.repository.basedir,
                             dwd.target.repository.basedir,
                             stdout=PIPE)[0]
@@ -492,11 +512,12 @@ class Svn(OperationalTest):
 
         from os import mkdir
         from os.path import join
+        from vcpx.tests import DEBUG
 
         drepo = join(self.TESTDIR, 'rename_delete')
         mkdir(drepo)
 
-        darcs = ExternalCommand(command=['darcs'], cwd=drepo, nolog=True)
+        darcs = ExternalCommand(command=['darcs'], cwd=drepo, nolog=not DEBUG)
 
         darcs.execute('init')
 
@@ -519,11 +540,12 @@ class Svn(OperationalTest):
 
         from os import mkdir
         from os.path import join
+        from vcpx.tests import DEBUG
 
         drepo = join(self.TESTDIR, 'rename_delete_dir')
         mkdir(drepo)
 
-        darcs = ExternalCommand(command=['darcs'], cwd=drepo, nolog=True)
+        darcs = ExternalCommand(command=['darcs'], cwd=drepo, nolog=not DEBUG)
 
         darcs.execute('init')
 
@@ -554,6 +576,7 @@ class CvsOrderTest(OperationalTest):
 
         from os import mkdir, getcwd
         from os.path import join, exists
+        from vcpx.tests import DEBUG
 
         super(CvsOrderTest, self).setUp()
 
@@ -564,29 +587,31 @@ class CvsOrderTest(OperationalTest):
             cvscmd = ['cvs', '-d', repodir]
             mkdir(basedir)
             mkdir(repodir)
-            ExternalCommand(cwd=self.TESTDIR, nolog=True, command=cvscmd).execute(
-                'init')
-
             startdir = join(basedir, 'start')
             mkdir(startdir)
+
+            cvs = ExternalCommand(cwd=startdir, nolog=not DEBUG, command=cvscmd)
+            cvs.execute('init')
+            self.failIf(cvs.exit_status, "%r failed" % cvs._last_command)
+
             open(join(startdir, 'foo'), "w").close()
 
-            ExternalCommand(cwd=startdir, nolog=True, command=cvscmd).execute(
-                'import', '-m', 'one', 'test', 'test', 'test1')
+            cvs.execute('import', '-m', 'one', 'test', 'test', 'test1')
+            self.failIf(cvs.exit_status, "%r failed" % cvs._last_command)
 
             workdir = join(basedir, 'work')
-            ExternalCommand(cwd=startdir, nolog=True, command=cvscmd).execute(
-                'checkout', '-d', workdir, 'test')
+            cvs.execute('checkout', '-d', workdir, 'test')
+            self.failIf(cvs.exit_status, "%r failed" % cvs._last_command)
 
             bardir = join(workdir, 'bar')
             mkdir(bardir)
             baz = join(bardir, 'baz')
             open(baz, "w").close()
 
-            ExternalCommand(cwd=workdir, nolog=True, command=cvscmd).execute(
-                'add', bardir, baz)
-            ExternalCommand(cwd=workdir, nolog=True, command=cvscmd).execute(
-                'commit', '-m', 'two')
+            cvs.execute('add', bardir, baz)
+            self.failIf(cvs.exit_status, "%r failed" % cvs._last_command)
+            cvs.execute('commit', '-m', 'two')
+            self.failIf(cvs.exit_status, "%r failed" % cvs._last_command)
 
     def testCvsConvertDirectoryAddToBazaarng(self):
         """Test that we can handle directory adds in the cvs module to bzr."""
@@ -611,3 +636,82 @@ class CvsOrderTest(OperationalTest):
 
         t = Tailorizer("svndirtest", self.config)
         t()
+
+
+class CvsReappearedDirectory(OperationalTest):
+    """Test problems with resurrected directories."""
+
+    def setUp(self):
+        """Create a CVS repository that has the difficult history."""
+
+        from os import mkdir, getcwd
+        from os.path import join, exists
+        from time import sleep
+        from shutil import rmtree
+        from vcpx.tests import DEBUG
+
+        super(CvsReappearedDirectory, self).setUp()
+
+        repodir = join(self.TESTDIR, 'cvsresurdirtest.cvsrepo')
+        basedir = join(self.TESTDIR, 'cvsresurdirtest')
+
+        if not exists(repodir):
+            mkdir(basedir)
+            mkdir(repodir)
+            startdir = join(basedir, 'start')
+            mkdir(startdir)
+
+            cvs = ExternalCommand(cwd=startdir, nolog=not DEBUG, command=['cvs', '-d', repodir])
+            cvs.execute('init')
+            self.failIf(cvs.exit_status, "%r failed" % cvs._last_command)
+
+            open(join(startdir, 'foo'), "w").close()
+
+            cvs.execute('import', '-m', 'one', 'test', 'test', 'test1')
+            self.failIf(cvs.exit_status, "%r failed" % cvs._last_command)
+
+            workdir = join(basedir, 'work')
+            cvs.execute('checkout', '-d', workdir, 'test')
+            self.failIf(cvs.exit_status, "%r failed" % cvs._last_command)
+
+            cvs = ExternalCommand(cwd=workdir, nolog=not DEBUG, command=['cvs'])
+            bardir = join(workdir, 'bar')
+            mkdir(bardir)
+            baz = join(bardir, 'baz')
+            open(baz, "w").close()
+
+            cvs.execute('add', bardir, baz)
+            self.failIf(cvs.exit_status, "%r failed" % cvs._last_command)
+            cvs.execute('commit', '-m', 'two', baz)
+            self.failIf(cvs.exit_status, "%r failed" % cvs._last_command)
+            sleep(1)
+
+            cvs.execute('rm', '-f', baz)
+            self.failIf(cvs.exit_status, "%r failed" % cvs._last_command)
+            cvs.execute('commit', '-m', 'three', baz)
+            self.failIf(cvs.exit_status, "%r failed" % cvs._last_command)
+            cvs.execute('update', '-dP')
+            self.failIf(cvs.exit_status, "%r failed" % cvs._last_command)
+            sleep(1)
+
+            mkdir(bardir)
+            again = join(bardir, 'again')
+            open(again, "w").close()
+
+            cvs.execute('add', bardir, again)
+            self.failIf(cvs.exit_status, "%r failed" % cvs._last_command)
+            cvs.execute('commit', '-m', 'four', again)
+            self.failIf(cvs.exit_status, "%r failed" % cvs._last_command)
+
+    def testCvsReappearedDirectoryToSubversion(self):
+        """Test that we can handle resurrected cvs directory to svn."""
+
+        from vcpx.tests import DEBUG
+
+        t = Tailorizer("svnresurdirtest", self.config)
+        t()
+
+        svnls = ExternalCommand(nolog=not DEBUG, command=['svn', 'ls'])
+        manifest = svnls.execute('file://%s/cvsresurdirtest.svnrepo/test/bar' % self.TESTDIR,
+                                 stdout=PIPE)[0]
+        self.assertEqual(manifest.read(), "again\n")
